#include <avr/interrupt.h>
#include <avr/io.h>

#include "RFM12.h"
//#include "Display.h"

#define WAIT_SDO_HI() 	while(~PINB&(1<<6))
#define WAIT_IRQ_LOW() 	while(PINB&(1<<2))

//////////////////////////////////////////////////////////
//UP1: Ports des ATMega32 initialisieren				//
//////////////////////////////////////////////////////////
//Legt die Datenrichtung fr Port B fest				//
//////////////////////////////////////////////////////////
void RF12_PORT_INIT()
{
	//DDR fr Out festlegen
	SEL_OUTPUT();
	SDI_OUTPUT();
	SCK_OUTPUT();
	DATA_OUT();
	LEDG_OUTPUT();
	LEDR_OUTPUT();

	//DDR fr In festlegen
	IRQ_IN();
	SDO_INPUT();

	//Def. Zustnde
	HI_SEL();
	HI_SDI();
	LOW_SCK();
}

//////////////////////////////////////////////////////////
//UP2: Befehl fr RF-Modul 								//
//////////////////////////////////////////////////////////
//Gibt dem RF Modul Befehle zum:						//
//	1. Lesen des Statusregisters		0x0000			//
//	2. Lesen des FIFO-Speichers			0xB000			//
//	3. Senden von Daten					0xB8xx			//
//	4. Beschreiben der Register			s.UP3			//
//////////////////////////////////////////////////////////
unsigned int RF12_WRT_CMD(unsigned int aCmd)
{
	unsigned char i;
	unsigned int temp;
	LOW_SCK();
//	LOW_SDI();
	LOW_SEL();
	
	for(i=0;i<16;i++)
	{
		temp<<=1;
		if(SDO_HI())
		{
			temp|=0x0001;
		}
		LOW_SCK();
		
		if(aCmd&0x8000)
		{
			HI_SDI();

		}
	
		else
		{
			LOW_SDI();
		}
		HI_SCK();
		aCmd<<=1;
		
	}
	LOW_SCK();
	
	HI_SEL();
	
	return(temp);
}

//////////////////////////////////////////////////////////
//UP3: Register von TX und RX konfiguieren				//
//////////////////////////////////////////////////////////
//Gemeinsame Registerinhalte fr Senden/Empfangen		//
//////////////////////////////////////////////////////////
void RF12_INIT()
{
	//Configuration Setting
	RF12_WRT_CMD(0x80EF);		//EL,EF,868band,12.0pF			V
	//Frequency Setting
	RF12_WRT_CMD(0xA000+1500);	//868MHz						V
	//Data Rate
	RF12_WRT_CMD(0xC614);		//19kbps						V
	//Data Filter
	RF12_WRT_CMD(0xC2AC);		//AL,!ml,DIG,DQD4				?
	//FIFO/Reset Mode
	RF12_WRT_CMD(0xCA81);		//FIFO8,SYNC,!ff,DR				X
	//Automatic Frequency Control
	RF12_WRT_CMD(0xC4C3);		//@PWR,NO RSTRIC,!st,!fi,OE,EN	X
	//Wake Up Timer
	RF12_WRT_CMD(0xE000);		//NOT USE						X
	//Low Duty Cycle
	RF12_WRT_CMD(0xC800);		//NOT USE						X
	RF12_WRT_CMD(0xC0E0);

}

//////////////////////////////////////////////////////////
//UP3: Register von TX 		konfiguieren				//
//////////////////////////////////////////////////////////
//Spezielle Registerinhalte fr das Senden				//
//////////////////////////////////////////////////////////
void RF12_INIT_TX()
{
	//Powermanagement
	RF12_WRT_CMD(0x8239);		//!er,!ebb,ET,ES,EX,!eb,!ew,DC	V
	//TX Configuration Control
	RF12_WRT_CMD(0x9800);		//!mp,9810=30kHz,MAX OUT		X
	//RX Control
	RF12_WRT_CMD(0x95C8);		//VDI,FAST,134kHz,0dBm,-103dBm 95A0	?
}

//////////////////////////////////////////////////////////
//UP5: Register von RX konfiguieren						//
//////////////////////////////////////////////////////////
//Spezielle Registerinhalte fr das Empfangen			//
//////////////////////////////////////////////////////////
void RF12_INIT_RX()
{
	//Powermanagement
	RF12_WRT_CMD(0x82D9);//!er,!ebb,ET,ES,EX,!eb,!ew,DC			V
	//TX Configuration Control
	RF12_WRT_CMD(0x9850);//!mp,9810=30kHz,MAX OUT				X
	//RX Control
	RF12_WRT_CMD(0b1001000100100000);										//?
}

//////////////////////////////////////////////////////////
//UP6: Statusregister auslesen							//
//////////////////////////////////////////////////////////
//UP zum Debugging										//
//////////////////////////////////////////////////////////
void RF12_READ_REG()
{
	int i,r0,r1,r2;
	r0=r1=r2=0;
	LOW_SDI();
	LOW_SEL();
	for(i=0;i<5;i++);

	
	for(i=0;i<8;i++)//X0...X7
	{
		HI_SCK();
		//Register<<=1;
		if(SDO_HI())
		{
			r0|=0x0001;
		}
		LOW_SCK();
		r0<<=1;
	}
	r0>>=1;

	for(i=0;i<8;i++)//X8...X
	{
		HI_SCK();
		//Register<<=1;
		if(SDO_HI())
		{
			r1|=0x0001;
		}
		LOW_SCK();
		r1<<=1;
	}
	r1>>=1;

	for(i=0;i<4;i++)
	{
		HI_SCK();
		//Register<<=1;
		if(SDO_HI())
		{
			r2|=0x0001;
		}
		LOW_SCK();
		r2<<=1;
   	}
	r2>>=1;

	HI_SEL();
	//Anzeige(r0,r1,r2);
	//lcd_write_cmd(0x80);

}

//////////////////////////////////////////////////////////
//UP7: Senden											//
//////////////////////////////////////////////////////////
//														//
//////////////////////////////////////////////////////////
void RF12_SEND(unsigned char Byte)
{
	while(PINB&(1<<2));//wait for previously TX over
	{
		LOW_SEL();
		WAIT_SDO_HI();
		RF12_WRT_CMD(0xB800+Byte);		//B8xx: Daten (xx) senden
	}
}

//////////////////////////////////////////////////////////
//UP8: Empfangen										//
//////////////////////////////////////////////////////////
//														//
//////////////////////////////////////////////////////////
unsigned int RF12_RECEIVE(void)
{
	unsigned int FIFO_data;
	
	
	/*//Flankenerkennung IRQ
	Merker = PINB&(1<<2);

	while(Merker==0);
	{
		Merker = PINB&(1<<2);
	}

	while(Merker!=0);
	{
		Merker = PINB&(1<<2);
	}*/


	RF12_WRT_CMD(0x0000);
	LOW_SEL();
	WAIT_SDO_HI();
	//WAIT_IRQ_LOW();
	FIFO_data=RF12_WRT_CMD(0xB000);	//B0xx: FIFO Daten auslesen
	
	return(FIFO_data & 0x00FF);
}
